/* Terraform - (C) 1997_1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111_1307, USA.
 */


#include "TFDialogOptions.h"
#include "TFOptions.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogOptions::TFDialogOptions (HeightField *HF, HeightFieldDraw *HFD)
	 : GuiDialogOAC ("Terraform Options Dialog", NULL),
	   d_frame1 ("Global Draw Options"), 
	   d_frame2("Height Field Draw Options"),
	   d_frame3("Fast Wireframe Options"),
	   d_frame4("POV Render Options"),
	   d_tabTitle1 ("Global"),
	   d_tabTitle2 ("Height Field View"),
	   d_tabTitle3 ("Fast Wireframe"),
	   d_tabTitle4 ("PovRay"),
	   d_msgScale ("Display Scale"),
	   d_msgSync ("Sync Rate (in lines)"),
	   d_msgResolution ("Wireframe Resolution"),
	   d_msgYScale ("Y Scale Factor"),
	   d_msgSealevel ("Sealevel"),
	   d_msgFillSea ("Sea is Filled"),
	   d_msgDrawDerived ("Draw derived data"),
	   d_msgFastResolution ("Half Wireframe Resolution"),
	   d_msgFastYscale ("Half Y Scale Factor"),
	   d_msgFastZrot ("Enable Z-axis rotation"),
	   d_msgPOVfileName ("POV template file"),
	   d_msgPOVwidth ("Render width"),
	   d_msgPOVyscale ("Half Y scale"),
	   d_msgPOVkeepFiles ("Keep generated files?"),
	   d_vBox1 (TRUE, 2),
	   d_vBox2 (TRUE, 2),
	   d_vBox3 (TRUE, 2),
	   d_vBox4 (TRUE, 2),
	   d_hBox1_1 (TRUE, 5),
	   d_hBox1_2 (TRUE, 5),
	   d_hBox1_3 (TRUE, 5),
	   d_hBox1_4 (TRUE, 5),
	   d_hBox2_1 (TRUE, 5),
	   d_hBox2_2 (TRUE, 5),
	   d_hBox2_3 (TRUE, 5),
	   d_hBox3_1 (TRUE, 5),
	   d_hBox3_2 (TRUE, 5),
	   d_hBox3_3 (TRUE, 5),
	   d_hBox4_1 (TRUE, 5),
	   d_hBox4_2 (TRUE, 5),
	   d_hBox4_3 (TRUE, 5),
	   d_hBox4_4 (TRUE, 5),
	   d_adjScale (1.0, 1.0, 3.0, 1.0),
	   d_adjSync (1, 1, 5000, 1),
	   d_adjResolution (5.0, 1.0, 10.0, 1.0),
	   d_adjYScale (0.33, 0, 1, 0.01, 10, 0),
	   d_adjSealevel (0.33, 0, 1, 0.01, 10, 0),
	   d_adjPOVwidth (400, 100, 1000, 1, 10, 0),
	   d_hsScale (d_adjScale),
	   d_hsSync (d_adjSync),
	   d_hsResolution (d_adjResolution),
	   d_hsYScale (d_adjYScale),
	   d_hsSealevel (d_adjSealevel),
	   d_hsPOVwidth (d_adjPOVwidth)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogOptions\n");

	setHFobjs (HF, HFD);

	buildDialogWindow ();
	updateSliders ();
}


/*
 *  destructor: clean up
 */
TFDialogOptions::~TFDialogOptions ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogOptions\n");
}


/*
 *  setHFobjs: set the HF obejcts we operate on 
 */
void TFDialogOptions::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogOptions::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogOptions::setHFobjs");

	d_HF = HF;
	d_HFD = HFD;
}


/*
 *  updateSliders: update the sliders with new/changed values from TFOptions
 */
void TFDialogOptions::updateSliders ()
{
	d_cbFillSea.set_active (TFOptions::s_fillSea);
	d_adjScale.set_value (TFOptions::s_scale); 
	d_adjSync.set_value (TFOptions::s_syncRate);
	d_adjResolution.set_value (TFOptions::s_resolution); 
	d_adjYScale.set_value (TFOptions::s_yscale); 
	d_adjSealevel.set_value (TFOptions::s_sealevel); 
	d_cbFastResolution.set_active (TFOptions::s_fastHalfRes);
	d_cbFastYscale.set_active (TFOptions::s_fastHalfYscale);
	d_cbFastZrot.set_active (TFOptions::s_fastZrot);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogOptions::fillVBox ()
{
	this->get_vbox()->pack_start (d_notebook, TRUE, TRUE, 0);
	d_frame1.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame2.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame3.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame4.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_notebook.append_page (d_frame1, d_tabTitle1);
	d_notebook.append_page (d_frame2, d_tabTitle2);
	d_notebook.append_page (d_frame3, d_tabTitle3);
	d_notebook.append_page (d_frame4, d_tabTitle4);
	d_notebook.show ();

	buildDrawOptionsFrame ();
	buildHFDrawOptionsFrame ();
	buildFastWireOptionsFrame ();
	buildPOVOptionsFrame ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogOptions::buildDialogWindow ()
{
	this->set_usize (350, 250);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


/*
 *  buildDrawOptionsFrame: frame for the global draw options
 */
void TFDialogOptions::buildDrawOptionsFrame ()
{
	d_msgScale.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_1.pack_start (d_msgScale, TRUE, TRUE, s_HBOff);
	d_msgScale.show ();
        d_hsScale.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsScale.set_digits (0);
        d_hsScale.set_draw_value (TRUE);
	d_hBox1_1.pack_start (d_hsScale, TRUE, TRUE, s_HBOff);
        d_hsScale.show ();
	d_vBox1.pack_start (d_hBox1_1, TRUE, TRUE, s_VBOff);
	d_hBox1_1.show ();

	d_msgSync.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_2.pack_start (d_msgSync, TRUE, TRUE, s_HBOff);
	d_msgSync.show ();
        d_hsSync.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsSync.set_digits (0);
        d_hsSync.set_draw_value (TRUE);
	d_hBox1_2.pack_start (d_hsSync, TRUE, TRUE, s_HBOff);
        d_hsSync.show ();
	d_vBox1.pack_start (d_hBox1_2, TRUE, TRUE, s_VBOff);
	d_hBox1_2.show ();

	d_msgFillSea.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_3.pack_start (d_msgFillSea, TRUE, TRUE, s_HBOff);
	d_msgFillSea.show ();
	d_cbFillSea.set_active (TRUE); 
	d_hBox1_3.pack_start (d_cbFillSea, TRUE, TRUE, s_HBOff);
	d_cbFillSea.show ();
	d_vBox1.pack_start (d_hBox1_3, TRUE, TRUE, s_VBOff);
	d_hBox1_3.show ();

	d_msgDrawDerived.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_4.pack_start (d_msgDrawDerived, TRUE, TRUE, s_HBOff);
	d_msgDrawDerived.show ();
	d_cbDrawDerived.set_active (TRUE); 
	d_hBox1_4.pack_start (d_cbDrawDerived, TRUE, TRUE, s_HBOff);
	d_cbDrawDerived.show ();
	d_vBox1.pack_start (d_hBox1_4, TRUE, TRUE, s_VBOff);
	d_hBox1_4.show ();


	d_frame1.add (d_vBox1);
	d_vBox1.show ();
	d_frame1.show ();
}


/*
 *  buildHFDrawOptionsFrame: frame for the Height Field draw options
 */
void TFDialogOptions::buildHFDrawOptionsFrame ()
{
	d_msgResolution.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox2_1.pack_start (d_msgResolution, TRUE, TRUE, s_HBOff);
	d_msgResolution.show ();
        d_hsResolution.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsResolution.set_digits (0);
        d_hsResolution.set_draw_value (TRUE);
	d_hBox2_1.pack_start (d_hsResolution, TRUE, TRUE, s_HBOff);
        d_hsResolution.show ();
	d_vBox2.pack_start (d_hBox2_1, TRUE, TRUE, s_VBOff);
	d_hBox2_1.show ();

	d_msgYScale.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox2_2.pack_start (d_msgYScale, TRUE, TRUE, s_HBOff);
	d_msgYScale.show ();
        d_hsYScale.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsYScale.set_digits (2);
        d_hsYScale.set_draw_value (TRUE);
	d_hBox2_2.pack_start (d_hsYScale, TRUE, TRUE, s_HBOff);
        d_hsYScale.show ();
	d_vBox2.pack_start (d_hBox2_2, TRUE, TRUE, s_VBOff);
	d_hBox2_2.show ();

	d_msgSealevel.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox2_3.pack_start (d_msgSealevel, TRUE, TRUE, s_HBOff);
	d_msgSealevel.show ();
        d_hsSealevel.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsSealevel.set_digits (2);
        d_hsSealevel.set_draw_value (TRUE);
	d_hBox2_3.pack_start (d_hsSealevel, TRUE, TRUE, s_HBOff);
        d_hsSealevel.show ();
	d_vBox2.pack_start (d_hBox2_3, TRUE, TRUE, s_VBOff);
	d_hBox2_3.show ();

	d_frame2.add (&d_vBox2);
	d_vBox2.show ();
	d_frame2.show ();
}

/*
 *  buildFastDrawOptionsFrame: frame for the Height Field draw options
 */
void TFDialogOptions::buildFastWireOptionsFrame ()
{
	d_msgFastResolution.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox3_1.pack_start (d_msgFastResolution, TRUE, TRUE, s_HBOff);
	d_msgFastResolution.show ();
	d_cbFastResolution.set_active (TRUE);
	d_hBox3_1.pack_start (d_cbFastResolution, TRUE, TRUE, s_HBOff);
	d_cbFastResolution.show ();
	d_vBox3.pack_start (d_hBox3_1, TRUE, TRUE, s_VBOff);
	d_hBox3_1.show ();

	d_msgFastYscale.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox3_2.pack_start (d_msgFastYscale, TRUE, TRUE, s_HBOff);
	d_msgFastYscale.show ();
	d_cbFastYscale.set_active (TRUE);
	d_hBox3_2.pack_start (d_cbFastYscale, TRUE, TRUE, s_HBOff);
	d_cbFastYscale.show ();
	d_vBox3.pack_start (d_hBox3_2, TRUE, TRUE, s_VBOff);
	d_hBox3_2.show ();

	d_msgFastZrot.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox3_3.pack_start (d_msgFastZrot, TRUE, TRUE, s_HBOff);
	d_msgFastZrot.show ();
	d_cbFastZrot.set_active (FALSE);
	d_hBox3_3.pack_start (d_cbFastZrot, TRUE, TRUE, s_HBOff);
	d_cbFastZrot.show ();
	d_vBox3.pack_start (d_hBox3_3, TRUE, TRUE, s_VBOff);
	d_hBox3_3.show ();

	d_frame3.add (&d_vBox3);
	d_vBox3.show ();
	d_frame3.show ();

}


/*
 *  buildPOVOptionsFrame: build the frame for the Export->POV menu options
 */
void TFDialogOptions::buildPOVOptionsFrame ()
{
	d_msgPOVfileName.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_1.pack_start (d_msgPOVfileName, TRUE, TRUE, s_HBOff);
	d_msgPOVfileName.show ();
	d_enPOVfile.set_text (TFOptions::s_POVfile);
	d_hBox4_1.pack_start (d_enPOVfile, TRUE, TRUE, s_HBOff);
	d_enPOVfile.show ();
	d_vBox4.pack_start (d_hBox4_1, TRUE, TRUE, s_VBOff);
	d_hBox4_1.show ();

	d_msgPOVwidth.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_2.pack_start (d_msgPOVwidth, TRUE, TRUE, s_HBOff);
	d_msgPOVwidth.show ();
	d_adjPOVwidth.set_value (d_HF->getWidth());
	d_hBox4_2.pack_start (d_hsPOVwidth, TRUE, TRUE, s_HBOff);
	d_hsPOVwidth.show ();
        d_hsScale.set_digits (0);
	d_vBox4.pack_start (d_hBox4_2, TRUE, TRUE, s_VBOff);
	d_hBox4_2.show ();

	d_msgPOVyscale.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_3.pack_start (d_msgPOVyscale, TRUE, TRUE, s_HBOff);
	d_msgPOVyscale.show ();
	d_cbPOVhalfYscale.set_active (TRUE);
	d_hBox4_3.pack_start (d_cbPOVhalfYscale, TRUE, TRUE, s_HBOff);
	d_cbPOVhalfYscale.show ();
	d_vBox4.pack_start (d_hBox4_3, TRUE, TRUE, s_VBOff);
	d_hBox4_3.show ();

	d_msgPOVkeepFiles.set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_4.pack_start (d_msgPOVkeepFiles, TRUE, TRUE, s_HBOff);
	d_msgPOVkeepFiles.show ();
	d_cbPOVkeepFiles.set_active (FALSE);
	d_hBox4_4.pack_start (d_cbPOVkeepFiles, TRUE, TRUE, s_HBOff);
	d_cbPOVkeepFiles.show ();
	d_vBox4.pack_start (d_hBox4_4, TRUE, TRUE, s_VBOff);
	d_hBox4_4.show ();

	d_frame4.add (&d_vBox4);
	d_vBox4.show ();
	d_frame4.show ();

}


/*
 *  getValues: get all the values from the dialog inputs
 */
void TFDialogOptions::getValues (bool *fillSea, bool *drawDerived, int *scale, 
				int *syncRate, int *resolution, float *yscale, 
				float *sealevel, bool *fastHalfRes, 
				bool *fastHalfYscale, bool *fastZrot)
{
	*fillSea = d_cbFillSea.get_active ();
	*drawDerived = d_cbDrawDerived.get_active ();
	*scale = (int)(d_adjScale.get_value ()); 
	*syncRate = (int)(d_adjSync.get_value());
	*resolution = (int)(d_adjResolution.get_value ()); 
	*yscale= d_adjYScale.get_value (); 
	*sealevel = d_adjSealevel.get_value (); 
	*fastHalfRes = d_cbFastResolution.get_active ();
	*fastHalfYscale = d_cbFastYscale.get_active ();
	*fastZrot = d_cbFastZrot.get_active ();
}


void TFDialogOptions::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: update what's been changed and redraw if necessary
 */
void TFDialogOptions::buttonCallbackApply ()
{
	bool			redraw=FALSE;

	int			scale, 
				syncRate;
	bool			fillSea,
				drawDerived,
				fastHalfRes,
				fastHalfYscale,
				fastZrot; 

	int			resolution;
	float			yscale, 
				sealevel;

	getValues (&fillSea, &drawDerived, &scale, &syncRate, &resolution, &yscale, &sealevel,
		   &fastHalfRes, &fastHalfYscale, &fastZrot);

	if (fillSea != TFOptions::s_fillSea)
		{
		TFOptions::s_fillSea = fillSea;
		redraw = TRUE;
		}

	if (drawDerived != TFOptions::s_drawDerivedData)
		{
		TFOptions::s_drawDerivedData = drawDerived;
		redraw = TRUE;
		}

	if (scale != TFOptions::s_scale)	
		{
		TFOptions::s_scale = scale;
		redraw = TRUE;
		}

	if (syncRate != TFOptions::s_syncRate)
		{
		TFOptions::s_syncRate = syncRate;
		redraw = TRUE;
		}

	if (resolution != TFOptions::s_resolution)
		{
		TFOptions::s_resolution = resolution;
		redraw = TRUE;
		}

	if (yscale != TFOptions::s_yscale)
		{
		TFOptions::s_yscale = yscale;
		redraw = TRUE;
		}

	if (sealevel != TFOptions::s_sealevel)
		{
		TFOptions::s_sealevel = sealevel;
		redraw = TRUE;
		}

	d_HFD->updateParams ();

	TFOptions::s_fastHalfRes = fastHalfRes;
	TFOptions::s_fastHalfYscale = fastHalfRes;
	TFOptions::s_fastZrot = fastZrot;
	d_HFD->setFastOptions (fastHalfRes, fastHalfYscale, fastZrot);

	strcpy (TFOptions::s_POVfile, d_enPOVfile.get_text().c_str());
	TFOptions::s_POVwidth = (int)(d_adjPOVwidth.get_value());
	TFOptions::s_POVhalfYscale = d_cbPOVhalfYscale.get_active();
	TFOptions::s_POVkeepFiles = d_cbPOVkeepFiles.get_active ();

	if (redraw)
		d_HFD->draw ();
}


gint TFDialogOptions::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}




